@@ -20,6 +20,7 @@
 
 #include "fileundomanager.h"
 #include "fileundomanager_p.h"
+#include "clipboardupdater_p.h"
 #include "fileundomanager_adaptor.h"
 
 #include <kdatetime.h>
@@ -31,7 +32,7 @@
 #include <kio/jobuidelegate.h>
 #include <klocale.h>
 #include <kmessagebox.h>
-#include <kuiserverjobtracker.h>
+#include <kjobtrackerinterface.h>
 
 #include <QtDBus/QtDBus>
 
@@ -40,7 +41,7 @@
 using namespace KIO;
 
 static const char* undoStateToString(UndoState state) {
-    static const char* s_undoStateToString[] = { "MAKINGDIRS", "MOVINGFILES", "STATINGFILE", "REMOVINGDIRS", "REMOVINGLINKS" };
+    static const char* const s_undoStateToString[] = { "MAKINGDIRS", "MOVINGFILES", "STATINGFILE", "REMOVINGDIRS", "REMOVINGLINKS" };
     return s_undoStateToString[state];
 }
 
@@ -136,11 +137,12 @@ CommandRecorder::CommandRecorder(FileUndoManager::CommandType op, const KUrl::Li
   connect(job, SIGNAL(result(KJob*)),
           this, SLOT(slotResult(KJob*)));
 
-  if (op != FileUndoManager::Mkdir) {
+  // TODO whitelist, instead
+  if (op != FileUndoManager::Mkdir && op != FileUndoManager::Put) {
       connect(job, SIGNAL(copyingDone(KIO::Job*,KUrl,KUrl,time_t,bool,bool)),
               this, SLOT(slotCopyingDone(KIO::Job*,KUrl,KUrl,time_t,bool,bool)));
-      connect(job, SIGNAL(copyingLinkDone(KIO::Job *,KUrl,QString,KUrl)),
-              this, SLOT(slotCopyingLinkDone(KIO::Job *,KUrl,QString,KUrl)));
+      connect(job, SIGNAL(copyingLinkDone(KIO::Job*,KUrl,QString,KUrl)),
+              this, SLOT(slotCopyingLinkDone(KIO::Job*,KUrl,QString,KUrl)));
   }
 }
 
@@ -208,9 +210,14 @@ FileUndoManager *FileUndoManager::self()
     return &globalFileUndoManager->self;
 }
 
+
+// m_nextCommandIndex is initialized to a high number so that konqueror can
+// assign low numbers to closed items loaded "on-demand" from a config file
+// in KonqClosedWindowsManager::readConfig and thus maintaining the real
+// order of the undo items.
 FileUndoManagerPrivate::FileUndoManagerPrivate(FileUndoManager* qq)
     : m_uiInterface(new FileUndoManager::UiInterface()),
-      m_undoJob(0), m_nextCommandIndex(0), q(qq)
+      m_undoJob(0), m_nextCommandIndex(1000), q(qq)
 {
     m_syncronized = initializeFromKDesky();
     (void) new KIOFileUndoManagerAdaptor(this);
@@ -278,21 +285,23 @@ QString FileUndoManager::undoText() const
     if (d->m_commands.isEmpty())
         return i18n("Und&o");
 
-    FileUndoManager::CommandType t = d->m_commands.top().m_type;
-    if (t == FileUndoManager::Copy)
+    FileUndoManager::CommandType t = d->m_commands.last().m_type;
+    switch(t) {
+    case FileUndoManager::Copy:
         return i18n("Und&o: Copy");
-    else if (t == FileUndoManager::Link)
+    case FileUndoManager::Link:
         return i18n("Und&o: Link");
-    else if (t == FileUndoManager::Move)
+    case FileUndoManager::Move:
         return i18n("Und&o: Move");
-    else if (t == FileUndoManager::Rename)
+    case FileUndoManager::Rename:
         return i18n("Und&o: Rename");
-    else if (t == FileUndoManager::Trash)
+    case FileUndoManager::Trash:
         return i18n("Und&o: Trash");
-    else if (t == FileUndoManager::Mkdir)
+    case FileUndoManager::Mkdir:
         return i18n("Und&o: Create Folder");
-    else
-        assert(false);
+    case FileUndoManager::Put:
+        return i18n("Und&o: Create File");
+    }
     /* NOTREACHED */
     return QString();
 }
@@ -306,7 +315,7 @@ quint64 FileUndoManager::currentCommandSerialNumber() const
 {
     if(!d->m_commands.isEmpty())
     {
-        const UndoCommand& cmd = d->m_commands.top();
+        const UndoCommand& cmd = d->m_commands.last();
         assert(cmd.m_valid);
         return cmd.m_serialNumber;
     } else
@@ -316,7 +325,7 @@ quint64 FileUndoManager::currentCommandSerialNumber() const
 void FileUndoManager::undo()
 {
     // Make a copy of the command to undo before broadcastPop() pops it.
-    UndoCommand cmd = d->m_commands.top();
+    UndoCommand cmd = d->m_commands.last();
     assert(cmd.m_valid);
     d->m_current = cmd;
 
@@ -325,14 +334,14 @@ void FileUndoManager::undo()
 
     // Let's first ask for confirmation if we need to delete any file (#99898)
     KUrl::List fileCleanupStack;
-    QStack<BasicOperation>::Iterator it = opStack.begin();
+    BasicOperation::Stack::Iterator it = opStack.begin();
     for (; it != opStack.end() ; ++it) {
         BasicOperation::Type type = (*it).m_type;
         if (type == BasicOperation::File && d->m_current.m_type == FileUndoManager::Copy) {
             fileCleanupStack.append((*it).m_dst);
         }
     }
-    if (d->m_current.m_type == FileUndoManager::Mkdir) {
+    if (d->m_current.m_type == FileUndoManager::Mkdir || d->m_current.m_type == FileUndoManager::Put) {
         fileCleanupStack.append(d->m_current.m_dst);
     }
     if (!fileCleanupStack.isEmpty()) {
@@ -372,7 +381,7 @@ void FileUndoManager::undo()
         }
         else if (type == BasicOperation::Link)
         {
-            d->m_linkCleanupStack.prepend((*it).m_dst);
+            d->m_fileCleanupStack.prepend((*it).m_dst);
 
             removeBasicOperation = !d->m_current.isMoveCommand();
         }
@@ -383,6 +392,10 @@ void FileUndoManager::undo()
             ++it;
     }
 
+    if (d->m_current.m_type == FileUndoManager::Put) {
+        d->m_fileCleanupStack.append(d->m_current.m_dst);
+    }
+
     kDebug(1203) << "starting with" << undoStateToString(d->m_undoState);
     d->m_undoJob = new UndoJob(d->m_uiInterface->showProgressInfo());
     d->undoStep();
@@ -392,7 +405,7 @@ void FileUndoManagerPrivate::stopUndo(bool step)
 {
     m_current.m_opStack.clear();
     m_dirCleanupStack.clear();
-    m_linkCleanupStack.clear();
+    m_fileCleanupStack.clear();
     m_undoState = REMOVINGDIRS;
     m_undoJob = 0;
 
@@ -416,7 +429,7 @@ void FileUndoManagerPrivate::slotResult(KJob *job)
     }
     else if (m_undoState == STATINGFILE)
     {
-        BasicOperation op = m_current.m_opStack.top();
+        BasicOperation op = m_current.m_opStack.last();
         //kDebug(1203) << "stat result for " << op.m_dst;
         KIO::StatJob* statJob = static_cast<KIO::StatJob*>(job);
         time_t mtime = statJob->statResult().numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1);
@@ -483,7 +496,7 @@ void FileUndoManagerPrivate::stepMovingFiles()
 {
     if (!m_current.m_opStack.isEmpty())
     {
-        BasicOperation op = m_current.m_opStack.top();
+        BasicOperation op = m_current.m_opStack.last();
         BasicOperation::Type type = op.m_type;
 
         assert(op.m_valid);
@@ -525,10 +538,11 @@ void FileUndoManagerPrivate::stepMovingFiles()
         {
             kDebug(1203) << "file_move" << op.m_dst << op.m_src;
             m_currentJob = KIO::file_move(op.m_dst, op.m_src, -1, KIO::Overwrite | KIO::HideProgressInfo);
+            new KIO::ClipboardUpdater(m_currentJob, KIO::ClipboardUpdater::UpdateContent);
             m_undoJob->emitMoving(op.m_dst, op.m_src);
         }
 
-        m_current.m_opStack.pop();
+        m_current.m_opStack.removeLast();
         // The above KIO jobs are lowlevel, they don't trigger KDirNotify notification
         // So we need to do it ourselves (but schedule it to the end of the undo, to compress them)
         KUrl url(op.m_dst);
@@ -546,9 +560,9 @@ void FileUndoManagerPrivate::stepMovingFiles()
 void FileUndoManagerPrivate::stepRemovingLinks()
 {
     kDebug(1203) << "REMOVINGLINKS";
-    if (!m_linkCleanupStack.isEmpty())
+    if (!m_fileCleanupStack.isEmpty())
     {
-        KUrl file = m_linkCleanupStack.pop();
+        KUrl file = m_fileCleanupStack.pop();
         kDebug(1203) << "file_delete" << file;
         m_currentJob = KIO::file_delete(file, KIO::HideProgressInfo);
         m_undoJob->emitDeleting(file);
@@ -607,14 +621,14 @@ void FileUndoManagerPrivate::slotPush(QByteArray data)
 
 void FileUndoManagerPrivate::pushCommand(const UndoCommand& cmd)
 {
-    m_commands.push(cmd);
+    m_commands.append(cmd);
     emit q->undoAvailable(true);
     emit q->undoTextChanged(q->undoText());
 }
 
 void FileUndoManagerPrivate::slotPop()
 {
-    m_commands.pop();
+    m_commands.removeLast();
     emit q->undoAvailable(q->undoAvailable());
     emit q->undoTextChanged(q->undoText());
 }
